@@ -250,16 +250,21 @@ MPtype <- function(MPs=NA) {
   
   Data <- DLMtool::SimulatedData
   dims <- dim(Data@Ind)
-  # Data@RInd <- array(Data@Ind, dim=c(dims[1],3,dims[2]))
+  n.ind <- 5
+  Data@AddInd <- array(Data@Ind, dim=c(dims[1],n.ind,dims[2]))
+  Data@AddIndV <- array(1, dim=c(dims[1],n.ind,dims[2])) 
+  Data@AddIndType <- rep(1, n.ind)
+  Data@AddIunits <- rep(1, n.ind)
+  Data@CV_AddInd <- array(0.1, dim=c(dims[1],n.ind,dims[2])) 
   
   runMPs <- applyMP(Data, MPs, reps = 2, nsims=1, silent=TRUE)
   recs <- runMPs[[1]]
   
   type <- rep("NA", length(MPs))
   rec <- rep("", length(MPs))
-  rectypes <- c("TAE", "Spatial", "SL")
+  rectypes <- c("TAE", "Spatial", "SL", "Discards")
   for (mm in seq_along(recs)) {
-    Effort <- Spatial <- Selectivity <- FALSE
+    Effort <- Spatial <- Selectivity <- Discards<- FALSE
     output <- length(recs[[mm]]$TAC) > 0 
     names <- names(recs[[mm]])
     names <- names[!names %in% c("TAC", "Spatial")]
@@ -276,8 +281,9 @@ MPtype <- function(MPs=NA) {
       if (any(is.finite(recs[[mm]]$LR5)) | any(is.finite(recs[[mm]]$LFR)) | any(is.finite(recs[[mm]]$HS)) |
           any(is.finite(recs[[mm]]$Rmaxlen)) | any(is.finite(recs[[mm]]$L5)) | any(is.finite(recs[[mm]]$LFS)) |
           any(is.finite(recs[[mm]]$Vmaxlen))) Selectivity <- TRUE
+      if (any(is.finite(recs[[mm]]$DR)) | any(is.finite(recs[[mm]]$Fdisc))) Discards <- TRUE
       
-      dorecs <- rectypes[c(Effort, Spatial, Selectivity)]
+      dorecs <- rectypes[c(Effort, Spatial, Selectivity, Discards)]
       thisrec <- dorecs
       type[mm] <- "Input"
       
